// package ch.epfl.compnet;
import java.util.*;
import java.io.*;
import java.net.*;

public class TCPServer {
    private static Map<String, Integer> getOccurrences(String message) {
        Map<String, Integer> occurrences = new TreeMap<String, Integer>();
        String delimiterRegexp = "[^a-zA-Z]+";
        Scanner fileScan = new Scanner(message).useDelimiter(delimiterRegexp);
        while(fileScan.hasNext()){
            String word = fileScan.next();
            word = word.toLowerCase();
            Integer oldCount = occurrences.get(word);
            if ( oldCount == null ) {
                oldCount = 0;
            }
            occurrences.put(word, oldCount + 1);
        }
        fileScan.close();
        return occurrences;
    }

    private static void printMap(Map<String, Integer> occurrences) {
        int num_values = occurrences.size();

        System.out.println("There are " + num_values + " unique words in the document \n");

        for (String key: occurrences.keySet()) {
            String word = key.toString();
            String times = occurrences.get(key).toString();

            System.out.println(word + ": " + times);
        }
    }

    private static void sendResponse(DataOutputStream outToClient, String message) {

        // Perform word-occurrence stats
        Map<String, Integer> occurrences = getOccurrences(message);
        printMap(occurrences);
    }

    private static void handleConnection(Socket connectionSocket) {
        DataInputStream inFromClient = null;
        DataOutputStream outToClient = null;

        try {
            // Open the input-output streams
            inFromClient = new DataInputStream(connectionSocket.getInputStream());
            outToClient = new DataOutputStream(connectionSocket.getOutputStream());

            do {
                System.out.println("Waiting to receive a file...");

                // Read the file contents into message
                byte[] bytearray = new byte[1024];
                int idx = 0;
                while (true) {
                    bytearray[idx++] = inFromClient.readByte();

                    // Process the special character
                    if (((char) bytearray[idx - 1]) == '|') {
                        break;
                    }
                }
                String message = new String(bytearray);
                System.out.println(message);

                // Call the response handler
                sendResponse(outToClient, message);
            } while (true);
        } catch (IOException ioex) {
            System.out.println("Failed to handle connection : " + ioex.getMessage());
        }
    }

    public static void main(String args[]) {
        ServerSocket welcomeSocket = null;
        Socket connectionSocket = null;

        try {
            // Create a socket that listens to port 6789
            welcomeSocket = new ServerSocket(6789);

            // Get a new connection
            System.out.println("Waiting for a new client...");
            connectionSocket = welcomeSocket.accept();

            // pass the connection socket to the handler
            System.out.println("Handling new client...");
            handleConnection(connectionSocket);
        } catch (IOException ioex) {
            System.out.println("Failed to open welcomeSocket : " + ioex.getMessage());
        }
    }
}